@@ -34,6 +34,7 @@
 #include "editor-page-private.h"
 #include "editor-sidebar-model-private.h"
 #include "editor-session-private.h"
+#include "editor-source-map.h"
 #include "editor-source-view.h"
 #include "editor-utils-private.h"
 
@@ -57,6 +58,26 @@ G_DEFINE_TYPE (EditorPage, editor_page, GTK_TYPE_WIDGET)
 
 static GParamSpec *properties [N_PROPS];
 
+static gboolean
+get_tab_info (EditorPage  *self,
+              AdwTabView **tab_view,
+              AdwTabPage **tab_page)
+{
+  GtkWidget *widget;
+
+  *tab_view = NULL;
+  *tab_page = NULL;
+
+  widget = gtk_widget_get_ancestor (GTK_WIDGET (self), ADW_TYPE_TAB_VIEW);
+  if (widget == NULL)
+    return FALSE;
+
+  *tab_view = ADW_TAB_VIEW (widget);
+  *tab_page = adw_tab_view_get_page (*tab_view, GTK_WIDGET (self));
+
+  return *tab_view != NULL && *tab_page != NULL;
+}
+
 static void
 on_draw_spaces_changed (EditorPage  *self,
                         const gchar *key,
@@ -208,7 +229,7 @@ editor_page_document_notify_busy_cb (EditorPage     *self,
   if (!busy)
     _editor_widget_hide_with_fade (GTK_WIDGET (self->progress_bar));
   else
-    gtk_widget_show (GTK_WIDGET (self->progress_bar));
+    gtk_widget_set_visible (GTK_WIDGET (self->progress_bar), TRUE);
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_BUSY]);
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CAN_SAVE]);
@@ -667,6 +688,11 @@ editor_page_dispose (GObject *object)
 {
   EditorPage *self = (EditorPage *)object;
 
+  g_cancellable_cancel (self->cancellable);
+  g_clear_object (&self->cancellable);
+
+  gtk_widget_dispose_template (GTK_WIDGET (self), EDITOR_TYPE_PAGE);
+
   g_clear_pointer (&self->progress_animation, editor_animation_stop);
   g_clear_pointer ((GtkWidget **)&self->box, gtk_widget_unparent);
 
@@ -882,6 +908,7 @@ editor_page_class_init (EditorPageClass *klass)
 
   g_type_ensure (EDITOR_TYPE_INFO_BAR);
   g_type_ensure (EDITOR_TYPE_SEARCH_BAR);
+  g_type_ensure (EDITOR_TYPE_SOURCE_MAP);
   g_type_ensure (EDITOR_TYPE_SOURCE_VIEW);
 }
 
@@ -890,6 +917,8 @@ editor_page_init (EditorPage *self)
 {
   GtkDropTarget *dest;
 
+  self->cancellable = g_cancellable_new ();
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
   editor_source_view_prepend_extra_menu (EDITOR_SOURCE_VIEW (self->view),
@@ -1013,10 +1042,10 @@ editor_page_is_active (EditorPage *self)
 
   g_return_val_if_fail (EDITOR_IS_PAGE (self), FALSE);
 
-  tab_view = ADW_TAB_VIEW (gtk_widget_get_ancestor (GTK_WIDGET (self), ADW_TYPE_TAB_VIEW));
-  tab_page = adw_tab_view_get_page (tab_view, GTK_WIDGET (self));
+  if (get_tab_info (self, &tab_view, &tab_page))
+    return tab_page == adw_tab_view_get_selected_page (tab_view);
 
-  return tab_page == adw_tab_view_get_selected_page (tab_view);
+  return FALSE;
 }
 
 gchar *
@@ -1101,9 +1130,8 @@ _editor_page_raise (EditorPage *self)
   title = editor_page_dup_title (self);
   g_debug ("Attempting to raise page: \"%s\"", title);
 
-  tab_view = ADW_TAB_VIEW (gtk_widget_get_ancestor (GTK_WIDGET (self), ADW_TYPE_TAB_VIEW));
-  tab_page = adw_tab_view_get_page (tab_view, GTK_WIDGET (self));
-  adw_tab_view_set_selected_page (tab_view, tab_page);
+  if (get_tab_info (self, &tab_view, &tab_page))
+    adw_tab_view_set_selected_page (tab_view, tab_page);
 }
 
 static void
@@ -1367,8 +1395,8 @@ editor_page_get_visual_position (EditorPage *self,
 gchar *
 editor_page_dup_position_label (EditorPage *self)
 {
-  guint line;
-  guint column;
+  guint line = 0;
+  guint column = 0;
 
   g_return_val_if_fail (EDITOR_IS_PAGE (self), NULL);
 
@@ -1409,6 +1437,10 @@ editor_page_get_can_discard (EditorPage *self)
 {
   g_return_val_if_fail (EDITOR_IS_PAGE (self), FALSE);
 
+  /* Still loading? Abandon it out of precaution */
+  if (gtk_source_buffer_get_loading (GTK_SOURCE_BUFFER (self->document)))
+    return TRUE;
+
   return editor_page_is_draft (self) &&
          !gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (self->document));
 }
@@ -1421,10 +1453,10 @@ _editor_page_position (EditorPage *self)
 
   g_return_val_if_fail (EDITOR_IS_PAGE (self), -1);
 
-  tab_view = ADW_TAB_VIEW (gtk_widget_get_ancestor (GTK_WIDGET (self), ADW_TYPE_TAB_VIEW));
-  tab_page = adw_tab_view_get_page (tab_view, GTK_WIDGET (self));
+  if (get_tab_info (self, &tab_view, &tab_page))
+    return adw_tab_view_get_page_position (tab_view, tab_page);
 
-  return adw_tab_view_get_page_position (tab_view, tab_page);
+  return -1;
 }
 
 gboolean
@@ -1706,3 +1738,19 @@ _editor_page_zoom_one (EditorPage *self)
 
   g_object_set (self->view, "font-scale", 0, NULL);
 }
+
+GCancellable *
+_editor_page_get_cancellable (EditorPage *self)
+{
+  g_return_val_if_fail (EDITOR_IS_PAGE (self), NULL);
+
+  return self->cancellable;
+}
+
+void
+editor_page_destroy (EditorPage *self)
+{
+  g_return_if_fail (EDITOR_IS_PAGE (self));
+
+  g_object_run_dispose (G_OBJECT (self));
+}
